﻿#target InDesign
var kScriptName = "Shadow-Matic";
var kVersion = "1.0.0";


// ShadowMatic.jsx
// Adobe InDesign automation script
// Developed by Premedia Systems, Inc.
// http://www.premediasystems.com 
// info@premediasystems.com

//////////////////////////////////////////////////////////////////////

// This script is designed to aid in the placement of a second 
// instance of an image with a multiply blending mode to create a
// shadow effect in InDesign.
//
// To use, select an image, and run the script. In the ensuing dialog,
// you will have the option of selecting the layers (if the image is a PSD)
// and clipping path to be applied to the "main" (i.e., top) image and
// the shadow (i.e, bottom) image. The script will duplicate the image, and 
// apply the appropriate layer visibility options and clipping path, set the
// blending mode of the bottom image container to "Multiply," and finally, 
// group the two image containers together.

//////////////////////////////////////////////////////////////////////

// Version history
// v 1.00 - 09/22/09 - Initial version

//////////////////////////////////////////////////////////////////////


var myDlg = new Window('dialog',kScriptName,undefined);

var theMainImageLayersCheckboxes = new Array();
var theMainImagePathRadioButtons = new Array();
var theShadowImageLayersCheckboxes = new Array();
var theShadowImagePathRadioButtons = new Array();

var theLayers = null;
var layersAreSupported = false;
var numLayers = 0;
	
main();


function main()
{
	if (app.documents.length == 0)
	{
		alert(kScriptName + " cannot proceed.\nNo documents open.");
		exit(0);
	}
	else
	{
		var myDoc = app.activeDocument;
		var theSelection = myDoc.selection;
		if (theSelection.length != 1)
		{
			alert(kScriptName + " cannot proceed.\nPlease select one image.");
			exit(0);
		}
		else
		{
			var theObj = theSelection[0];
			var theClass = theObj.constructor.name;
			if ((theClass == "Image") || (theClass == "EPS"))
			{
				var theImage = theObj;
			}
			else
			{
				if (theObj.allGraphics.length != 1)
				{
			alert(kScriptName + " cannot proceed.\nPlease select one image.");
					exit(0);
				}
				else
				{
					var theDeepObj = theObj.allGraphics[0];
					var theClass = theDeepObj.constructor.name;
					if ((theClass == "Image") || (theClass == "EPS") || (theClass == "PDF"))
					{
						var theImage = theDeepObj;
					}
					else
					{
			alert(kScriptName + " cannot proceed.\nPlease select one image.");
						exit(0);					
					}
				}
			}
		}
	}


	try{
		theLayers = theImage.graphicLayerOptions.graphicLayers
		numLayers = theLayers.length;
		layersAreSupported = (numLayers > 0); //a TIFF will have a non-null graphicLayers , but its count will be 0
	}
	catch(e){}  //if it fails the try, then it must be an image type which does not support layers.

	var thePaths = theImage.clippingPath.photoshopPathNames

	if ((numLayers < 2) && (thePaths.length == 0))
	{
		alert(kScriptName + " cannot proceed.\nThis image appears to not have the required number of layers and/or paths to proceed.\n\nThe image must be a PSD or PDF with two or more layers, or any image with one or more Photoshop paths.");
		exit(0);
	}

	buildWindow(theImage.itemLink.name, theImage.imageTypeName, theLayers, thePaths);


	var theResult = myDlg.show();

	if (theResult == -1)
	{
		//user cancelled
		exit(0);
	}
	else
	{
		//now, we need to handle the result - duplicate the image container
		//set the layer visibility and/or clipping path of each of the two images,
		//and finally, group the two together.
		
		//first, we will duplicate the image container
		var bottomImageContainer = theImage.parent;
		var topImageContainer = bottomImageContainer.duplicate();
		
		//First, we'll set the layer visibility for the top image, assuming that our image type supports layers
		if (theLayers != null)
			{
			for (var i=0; i < theMainImageLayersCheckboxes.length; i++) 
			{
				var errMsg = setLayerVisibility( topImageContainer.allGraphics[0], theMainImageLayersCheckboxes[i].text, theMainImageLayersCheckboxes[i].value);
				if (errMsg != "")
				{
					alert(errMsg);
				}
			}
			//and now for the bottom (shadow) image
			for (var i=0; i < theShadowImageLayersCheckboxes.length; i++) 
			{
				var errMsg = setLayerVisibility(bottomImageContainer.allGraphics[0], theShadowImageLayersCheckboxes[i].text, theShadowImageLayersCheckboxes[i].value);
				if (errMsg != "")
				{
					alert(errMsg);
				}
			}
		}

		//Here we check to see if there is a clipping path to be applied, and if so, we do it
		//Note that in both loops, we start at i = 1 rather than i = 0 since the first member of the array is the "None" option
			for (var i=1; i < theMainImagePathRadioButtons.length; i++)
			{
				if (theMainImagePathRadioButtons[i].value)
				{
					var errMsg = setClippingPath(topImageContainer.allGraphics[0], theMainImagePathRadioButtons[i].text);		
					if (errMsg != "")
					{
						alert(errMsg);
					}
				}
			}
			//and now for the bottom (shadow) image
			for (var i=1; i < theShadowImagePathRadioButtons.length; i++) 
			{	
				if (theShadowImagePathRadioButtons[i].value)
				{
					var errMsg = setClippingPath(bottomImageContainer.allGraphics[0], theShadowImagePathRadioButtons[i].text);		
					if (errMsg != "")
					{
						alert(errMsg);
					}
				}		
			}

		//now that we are done setting the layers and/or clipping paths, we need to set the shadow image to multiply
		var tempObj = bottomImageContainer; //.allGraphics[0];  //We want to pply the multiply blending mode to the container, not the image...
		tempObj.transparencySettings.blendingSettings.blendMode = BlendMode.MULTIPLY;
		
		//and finally, we need to group them together
		var theItems = new Array(bottomImageContainer,topImageContainer);
		var theGroup = myDoc.groups.add(theItems);
	}
}
//---------------------------------------------------------------------
function setLayerVisibility(theImage, theLayerName, theVisibility)
{
	var theLayer
	try 
	{
		theLayer = theImage.graphicLayerOptions.graphicLayers.itemByName(theLayerName);
	}
	catch(e)
	{
		return "Could not find layer '" + theLayerName + " in the image!\n'" + e.description
	}
	if (theLayer == null)
	{
		return "Could not find layer '" + theLayerName + " in the image!\n'" + e.description
	}
	//Now, it seems that we have the image layer with which to work
	theLayer.currentVisibility = theVisibility;
	return "";
}
//---------------------------------------------------------------------
function setClippingPath(theImage, thePathName)
{
	try 
	{
		theImage.clippingPath.appliedPathName = thePathName;
	}
	catch(e)
	{
		return "Could not find path '" + thePathName + " in the image!\n'" + e.description
	}	return "";
}
//---------------------------------------------------------------------
function buildWindow(theFileName, theFileType, theLayers, thePaths)
{
	myDlg.preferredSize.width = 500;
	myDlg.preferredSize.height = 200;
	myDlg.alignChildren = ['fill','fill'];
	myDlg.margins= [20,20,20,10];

// Properties for myDlg.Panel1
	myDlg.Panel1 = myDlg.add('panel',undefined,"Info");
	myDlg.Panel1.preferredSize.width = 480;
	myDlg.Panel1.preferredSize.height = 70;
	myDlg.Panel1.alignChildren = ['left',' '];

// Properties for myDlg.Panel1.Group1
	myDlg.Panel1.Group1 = myDlg.Panel1.add('group',undefined);

// Properties for myDlg.Panel1.Group1.StaticText1
	myDlg.Panel1.Group1.StaticText1 = myDlg.Panel1.Group1.add("statictext{text:'File Name:',justify:'right'}");
	myDlg.Panel1.Group1.StaticText1.alignment = ['right',' '];
	myDlg.Panel1.Group1.StaticText1.preferredSize.width = 70;
	myDlg.Panel1.Group1.StaticText1.graphics.font = ScriptUI.newFont(myDlg.Panel1.Group1.StaticText1.graphics.font.family,"BOLD",myDlg.Panel1.Group1.StaticText1.graphics.font.size);

// Properties for myDlg.Panel1.Group1.stFileName
	myDlg.Panel1.Group1.stFileName = myDlg.Panel1.Group1.add('statictext',undefined,theFileName);
	myDlg.Panel1.Group1.stFileName.alignment = ['left',' '];
	myDlg.Panel1.Group1.stFileName.preferredSize.width = 360;
	myDlg.Panel1.Group1.stFileName.graphics.font = ScriptUI.newFont(myDlg.Panel1.Group1.stFileName.graphics.font.family,"ITALIC",myDlg.Panel1.Group1.stFileName.graphics.font.size);

// Properties for myDlg.Panel1.Group2
	myDlg.Panel1.Group2 = myDlg.Panel1.add('group',undefined);
	myDlg.Panel1.Group2.spacing = 10;

// Properties for myDlg.Panel1.Group2.StaticText3
	myDlg.Panel1.Group2.StaticText3 = myDlg.Panel1.Group2.add("statictext{text:'File Type:',justify:'right'}");
	myDlg.Panel1.Group2.StaticText3.alignment = ['right',' '];
	myDlg.Panel1.Group2.StaticText3.preferredSize.width = 70;
	myDlg.Panel1.Group2.StaticText3.graphics.font = ScriptUI.newFont(myDlg.Panel1.Group2.StaticText3.graphics.font.family,"BOLD",myDlg.Panel1.Group2.StaticText3.graphics.font.size);

// Properties for myDlg.Panel1.Group2.stFileType
	myDlg.Panel1.Group2.stFileType = myDlg.Panel1.Group2.add('statictext',undefined,theFileType);
	myDlg.Panel1.Group2.stFileType.alignment = ['left',' '];
	myDlg.Panel1.Group2.stFileType.preferredSize.width = 360;
	myDlg.Panel1.Group2.stFileType.graphics.font = ScriptUI.newFont(myDlg.Panel1.Group2.stFileType.graphics.font.family,"ITALIC",myDlg.Panel1.Group2.stFileType.graphics.font.size);

// Properties for myDlg.Panel2
	myDlg.Panel2 = myDlg.add('panel',undefined,"Main");
	myDlg.Panel2.preferredSize.width = 480;
	myDlg.Panel2.preferredSize.height = 70;
	myDlg.Panel2.alignChildren = ['fill','fill'];
	myDlg.Panel2.margins= [10,15,10,10];
	myDlg.Panel2.orientation = "row";

// Properties for myDlg.Panel2.Group6
	myDlg.Panel2.Group6 = myDlg.Panel2.add('group',undefined);
	myDlg.Panel2.Group6.alignChildren = ['fill','fill'];

// Properties for myDlg.Panel2.Group6.mainLayersPanel
	myDlg.Panel2.Group6.mainLayersPanel = myDlg.Panel2.Group6.add('panel',undefined,"Layers");
	myDlg.Panel2.Group6.mainLayersPanel.alignChildren = ['left','top'];
	myDlg.Panel2.Group6.mainLayersPanel.margins= [20,20,20,0];
	
//Create the checkboxes
	if(layersAreSupported)
	{
		for (var i=0; i < theLayers.length; i++) {
			theMainImageLayersCheckboxes.push(myDlg.Panel2.Group6.mainLayersPanel.add('checkbox',undefined,theLayers[i].name));
		}
		//turn them all on
		handleMainLayersAll();
	}

// Properties for myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons = myDlg.Panel2.Group6.mainLayersPanel.add('group',undefined);
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.margins= [0,20,0,10];

// Properties for myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonAll
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonAll = myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.add('button',undefined,"All");
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonAll.preferredSize.width = 60;
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonAll.onClick = handleMainLayersAll;

// Properties for myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonNone
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonNone = myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.add('button',undefined,"None");
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonNone.preferredSize.width = 60;
	myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.buttonNone.onClick = handleMainLayersNone;

// Properties for myDlg.Panel2.mainPathsPanel
	myDlg.Panel2.mainPathsPanel = myDlg.Panel2.add('panel',undefined,"Clipping Path");
	myDlg.Panel2.mainPathsPanel.alignChildren = ['left','top'];
	myDlg.Panel2.mainPathsPanel.margins= [20,20,20,20];

// Properties for myDlg.Panel2.mainPathsPanel.RadioButton1
	rb1 = myDlg.Panel2.mainPathsPanel.add('radiobutton',undefined,"None");
	theMainImagePathRadioButtons.push(rb1);
	rb1.value = true;

	for (var i=0; i < thePaths.length; i++) {
		theMainImagePathRadioButtons.push(myDlg.Panel2.mainPathsPanel.add('radiobutton',undefined,thePaths[i]));
	}

// Properties for myDlg.Panel3
	myDlg.Panel3 = myDlg.add('panel',undefined,"Shadow");
	myDlg.Panel3.preferredSize.width = 480;
	myDlg.Panel3.preferredSize.height = 70;
	myDlg.Panel3.alignChildren = ['fill','fill'];
	myDlg.Panel3.margins= [10,15,10,10];
	myDlg.Panel3.orientation = "row";

// Properties for myDlg.Panel3.Group7
	myDlg.Panel3.Group7 = myDlg.Panel3.add('group',undefined);
	myDlg.Panel3.Group7.alignChildren = ['fill','fill'];

// Properties for myDlg.Panel3.Group7.shadowLayersPanel
	myDlg.Panel3.Group7.shadowLayersPanel = myDlg.Panel3.Group7.add('panel',undefined,"Layers");
	myDlg.Panel3.Group7.shadowLayersPanel.alignChildren = ['left','top'];
	myDlg.Panel3.Group7.shadowLayersPanel.margins= [20,20,20,0];

//Create the checkboxes
	//note that the layers array contains layer objects, not name stirng as the paths array does.
	//so, we need to call the name property in this case, but not with the paths	
	if(layersAreSupported)
	{		for (var i=0; i < theLayers.length; i++) {
			theShadowImageLayersCheckboxes.push(myDlg.Panel3.Group7.shadowLayersPanel.add('checkbox',undefined,theLayers[i].name));
		}

		//set them all to ON
		handleShadowLayersAll();
	}

// Properties for myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons = myDlg.Panel3.Group7.shadowLayersPanel.add('group',undefined);
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.margins= [0,20,0,10];
	


// Properties for myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonAll
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonAll = myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.add('button',undefined,"All");
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonAll.preferredSize.width = 60;
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonAll.onClick = handleShadowLayersAll;

// Properties for myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonNone
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonNone = myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.add('button',undefined,"None");
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonNone.preferredSize.width = 60;
	myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.buttonNone.onClick = handleShadowLayersNone;

// Properties for myDlg.Panel3.shadowPathsPanel
	myDlg.Panel3.shadowPathsPanel = myDlg.Panel3.add('panel',undefined,"Clipping Path");
	myDlg.Panel3.shadowPathsPanel.alignChildren = ['left','top'];
	myDlg.Panel3.shadowPathsPanel.margins= [20,20,20,20];

// Properties for myDlg.Panel3.shadowPathsPanel.RadioButton5
	rb2 = myDlg.Panel3.shadowPathsPanel.add('radiobutton',undefined,"None");
	theShadowImagePathRadioButtons.push(rb2);
	rb2.value = true;

	for (var i=0; i < thePaths.length; i++) {
		theShadowImagePathRadioButtons.push(myDlg.Panel3.shadowPathsPanel.add('radiobutton',undefined,thePaths[i]));
	}


// Properties for myDlg.Group3
	myDlg.Group3 = myDlg.add('group',undefined);
	myDlg.Group3.alignment = ['fill','bottom'];
	myDlg.Group3.margins= [0,0,0,0];
	myDlg.Group3.orientation = "row";
	myDlg.Group3.spacing = 0;

// Properties for myDlg.Group3.stEPSLayerWarning
	myDlg.Group3.stEPSLayerWarning = myDlg.Group3.add('statictext',undefined,"InDesign only supports layers in PSD and PDF images.");
	myDlg.Group3.stEPSLayerWarning.visible = !layersAreSupported;
	myDlg.Group3.stEPSLayerWarning.graphics.font = ScriptUI.newFont(myDlg.Group3.stEPSLayerWarning.graphics.font.family,"BOLDITALIC",9);
	myDlg.Group3.stEPSLayerWarning.graphics.foregroundColor = myDlg.Group3.stEPSLayerWarning.graphics.newPen(myDlg.Group3.stEPSLayerWarning.graphics.PenType.SOLID_COLOR,[1,0,0],1);

// Properties for myDlg.Group3.Group5
	myDlg.Group3.Group5 = myDlg.Group3.add('group',undefined);
	myDlg.Group3.Group5.alignment = ['right',' '];
	myDlg.Group3.Group5.margins= [0,0,0,0];
	myDlg.Group3.Group5.orientation = "row";
	myDlg.Group3.Group5.spacing = 0;

// Properties for myDlg.Group3.Group5.Button2
	myDlg.Group3.Group5.Button2 = myDlg.Group3.Group5.add('button',undefined,"Cancel");
	myDlg.Group3.Group5.Button2.onClick = handleCancelbutton;

// Properties for myDlg.Group3.Group5.Button1
	myDlg.Group3.Group5.Button1 = myDlg.Group3.Group5.add('button',undefined,"OK");
	myDlg.Group3.Group5.Button1.onClick = handleOKbutton;
	
// Properties for rapidDlg.StaticText6
	myDlg.StaticText6 = myDlg.add("statictext{text:'Copyright © 2009, Premedia Systems, Inc. All Rights Reserved',justify:'center'}");
	myDlg.StaticText6.alignment = ['fill',' '];
	myDlg.StaticText6.graphics.font = ScriptUI.newFont(myDlg.StaticText6.graphics.font.family,myDlg.StaticText6.graphics.font.style,9);
	
	
//turn on and off what we need to based on layer support in this file type	
	if (!layersAreSupported) // layers not supported in this file type (e.g., EPS, TIFF)
	{
		myDlg.Panel3.Group7.shadowLayersPanel.enabled = false;
		myDlg.Panel3.Group7.shadowLayersPanel.shadowLayersButtons.visible = false;
		myDlg.Panel2.Group6.mainLayersPanel.enabled = false;
		myDlg.Panel2.Group6.mainLayersPanel.mainLayersButtons.visible = false;
	}
	else
	{

	}

}

//---------------------------------------------------------------------
function handleOKbutton()
{
	myDlg.close(1);
}
//---------------------------------------------------------------------
function handleCancelbutton()
{
	myDlg.close(-1);
}

//---------------------------------------------------------------------
function handleMainLayersNone()
{
	for (var i=0; i < theMainImageLayersCheckboxes.length; i++) 
	{
		theMainImageLayersCheckboxes[i].value = false;
	}	
}

//---------------------------------------------------------------------
function handleMainLayersAll()
{
	for (var i=0; i < theMainImageLayersCheckboxes.length; i++) 
	{
		theMainImageLayersCheckboxes[i].value = true;
	}	
}

//---------------------------------------------------------------------
function handleShadowLayersNone()
{
	for (var i=0; i < theShadowImageLayersCheckboxes.length; i++) 
	{
		theShadowImageLayersCheckboxes[i].value = false;
	}	
}

//---------------------------------------------------------------------
function handleShadowLayersAll()
{
	for (var i=0; i < theShadowImageLayersCheckboxes.length; i++) 
	{
		theShadowImageLayersCheckboxes[i].value = true;
	}	
}